/*

    Meta-Environment - An environment for language prototyping.
    Copyright (C) 2000  Stichting Mathematisch Centrum, Amsterdam, The Netherlands. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    $Id: PathConstruction.java,v 1.9 2000/04/15 15:18:44 mdejonge Exp $
*/
package nl.cwi.gipe.stred;
import toolbus.aterm.*;

public class PathConstruction {
  public static Path getPath(ATerm t, int i) {
    return getPath(t, i, new Path());
  }

  public static Path getPath(ATerm t, int i, Path p) {
      //      System.err.println(Integer.toString(i));
    //debug
      //System.err.println("In getPath. t = "+ t.toString() +", i = "+Integer.toString(i)+", p = "+p.toString());
    AFun args = new AFun(ATerm.the_world,"args");
    AFun elems = new AFun(ATerm.the_world,"elems");
    AFun w_left = new AFun(ATerm.the_world,"w-left");
    AFun term = new AFun(ATerm.the_world,"term");
    if (t.getType() == ATerm.LIST) {
      // t is a list
      ATermList tl = (ATermList) t;
      ATerms ts = tl.getATerms();
      //System.err.println("PathConstruction.getPath: ts: "+ts.toString());
      ATermAppl t1 = (ATermAppl) ts.getFirst();
      //System.err.println("PathConstruction.getPath: t1: "+t1.toString());
      int j = Annotations.getLn(t1);
      //      System.err.println("The length of this term: "+Integer.toString(j));
      if (i>j) {
	//	if (ts.getNext().getType() != ATerm.ATERMS) {
	//	  return getPath(new ATermList(new ATerms(ts.getNext())), i-j, p.right());
	//	} else {
	return getPath(new ATermList(ATerm.the_world,ts.getNext()), i-j, p.right());
	//	}
      } else {
        if (t1.getFun().equals("appl")) {
	  return getPath(AsFix.acc(t1,args),i,p.down());
	} else if (t1.getFun().equals("list")) {
	  return getPath(AsFix.acc(t1,elems),i,p.down());
	} else if (t1.getFun().equals("term")) {
	  int leftlength = Annotations.getLn(AsFix.acc(t1,w_left));
	  if (i > leftlength) {
	      i -= leftlength;
	      ATerm termterm = AsFix.acc(t1,term);
	      int termlength = Annotations.getLn(termterm);
	      if (i > termlength) {
		  // Then the position is in the whitespace right from the term
		  return p;
	      } else {
		  // The position is in the term:
		  return getPath(termterm, i, p.down().right());
	      }
	  } else {
	      return p;
	  }
	} else if (t1.getFun().equals("lex") || t1.getFun().equals("var")) {
	  return p;
	} else {
				// "w" or "l"
	  return p.up();
	}
      }
    } else {
      // t is an application
      ATermAppl ta = (ATermAppl) t;
      if (ta.getFun().equals("appl")) {
	return getPath(AsFix.acc(ta,args),i,p.down());
      } else if (ta.getFun().equals("list")) {
	return getPath(AsFix.acc(ta,elems),i,p.down());
      } else if (ta.getFun().equals("term")) {
	  int leftlength = Annotations.getLn(AsFix.acc(ta,w_left));
	  if (i > leftlength) {
	      i -= leftlength;
	      ATerm termterm = AsFix.acc(ta,term);
	      int termlength = Annotations.getLn(termterm);
	      if (i > termlength) {
		  // Then the position is in the whitespace right from the term
		  return p;
	      } else {
		  // The position is in the term:
		  return getPath(termterm, i, p.down().right());
	      }
	  } else {
	      return p;
	  }
      } else {
				// "lex" and "var"
	return p;
      }
    }
  }
}
