/*

    Meta-Environment - An environment for language prototyping.
    Copyright (C) 2000  Stichting Mathematisch Centrum, Amsterdam, The Netherlands. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    $Id: PositionInformation.java,v 1.11 2000/04/15 15:18:44 mdejonge Exp $
*/
package nl.cwi.gipe.stred;
import toolbus.aterm.*;

public class PositionInformation {
  public static ATerm addToLen(ATerm t, int i) {
    // debug
    //    System.err.println("In addToLen. t = "+ t.toString()+", i = "+Integer.toString(i));
    Length l = new Length((ATermAppl) t.getAnnotation(Length.sym));
    int j = l.getInt();
    l.setInt(j+i);
    // setAnno is not implemented on ATerm. Let's cast t to a 
    // ATermAppl...
    //    ATermAppl at = (ATermAppl) t;
    //    at.setAnno(AsFixUtilities.replace(l,ts));
    t.setAnnotation(Length.sym,l);
    return t;
    
  }

    public static ATerm annotate(ATerm t) {
	// try {
	AFun args = new AFun(ATerm.the_world,"args");
	AFun elems = new AFun(ATerm.the_world,"elems");
	AFun w_left = new AFun(ATerm.the_world,"w-left");
	AFun term = new AFun(ATerm.the_world,"term");
	AFun w_right = new AFun(ATerm.the_world,"w-right");
	ATerm t1,t2,t3;
	int i;
	//	System.err.println("PositionInformation.annotate: t: "+t.toString());
	switch (t.getType()) {
	case ATerm.ATERMS:
	    ATerms tmp = (ATerms) t;
	    return new ATerms(ATerm.the_world,annotate(tmp.getFirst()),(tmp.getNext().isEmpty() == false) ? (ATerms) annotate(tmp.getNext()) : new ATerms(ATerm.the_world));
	case ATerm.APPL:
	    ATermAppl tmp2 = (ATermAppl) t;
	    String str = tmp2.getFun();
	    if (str.equals("term")) {
		t1 = annotate(AsFix.acc(tmp2,w_left));
		t2 = annotate(AsFix.acc(tmp2,term));
		t3 = annotate(AsFix.acc(tmp2,w_right));
		i = subtrmLen(t1) + subtrmLen(t2) + subtrmLen(t3);
		tmp2 = AsFix.repl(tmp2, w_left, t1);
		tmp2 = AsFix.repl(tmp2, term, t2);
		tmp2 = AsFix.repl(tmp2, w_right, t3);
		tmp2.setAnnotation(Length.sym,new Length(i));
		return tmp2;	
	    } else if (str.equals("appl")) {
		t1 = annotate((ATermList) AsFix.acc(tmp2,args));
		i = subtrmLen(t1);
		tmp2 = AsFix.repl(tmp2, args, t1);
		tmp2.setAnnotation(Length.sym,new Length(i));
		return tmp2;
	    } else if (str.equals("list")) {
		t1 = annotate((ATermList) AsFix.acc(tmp2, elems));
		i = subtrmLen(t1);
		//		t1.println(System.err);
		tmp2 = AsFix.repl(tmp2,elems,t1);
		tmp2.setAnnotation(Length.sym,new Length(i));
		return tmp2;
				//     } else if (str.equals("w")) {
				//	String ws = tmp2.getArgs().getFirst();
				//	i = ws.length();
				//	tmp2.setAnno(tmp2.getAnno().concat(new Length(i)));
				//	return tmp2;
	    } else {
				// str.equals("l")
		//		System.err.println("The string: "+tmp2.toString());
		ATermAppl fun = (ATermAppl) tmp2.getArgs().getFirst();
		String lex = fun.getFun();
		i = lex.length();
		tmp2.setAnnotation(Length.sym,new Length(i));
		return tmp2;
	    }
	case ATerm.LIST:
	    ATermList tl = (ATermList) t;
	    if (!tl.isEmpty()) {
		return new ATermList(ATerm.the_world,(ATerms)annotate(tl.getATerms()));
	    } else {
		return tl;
	    }
	default: //Should never happen. Should be an exception.
	    System.err.println("Unsupported ATerm type in PositionInformation.annotate");
	    System.exit(2);
    }
    //}
    //catch (SEException se) {
    //	System.exit(1);
    //}
	return null; //To keep the compiler happy
  }
  
  private static int subtrmLen(ATerm t) {
//    try {
    switch (t.getType()) {
    case ATerm.ATERMS:
      ATerms ts = (ATerms) t;
      return subtrmLen(ts.getFirst()) + ((! ts.getNext().isEmpty()) ? subtrmLen(ts.getNext()) : 0);
    case ATerm.APPL:
      Length l = new Length((ATermAppl) t.getAnnotation(Length.sym));
      return l.getInt();
    case ATerm.LIST:
      ATermList tl = (ATermList) t;
      if (!tl.isEmpty()) {
	  return subtrmLen(tl.getATerms());
      } else {
	  return 0;
      }
    default:
	System.err.println("Unsupported ATerm type in PositionInformation.subtrmLen");
	System.exit(2);
    }
  //  }
   // catch (SEException se) {
   // 	System.exit(1);
   // }
    return 0; // To keep compiler happy
  }    
}
	
